from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypeVar

from django.utils.translation import gettext as _
from edc_appointment.constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    INCOMPLETE_APPT,
    NEW_APPT,
    SKIPPED_APPT,
)
from edc_appointment.view_utils import AppointmentButton
from edc_utils import get_utcnow

if TYPE_CHECKING:
    from edc_visit_tracking.model_mixins import VisitModelMixin

    RelatedVisitModel = TypeVar("RelatedVisitModel", bound=VisitModelMixin)

__all__ = ["TimepointStatusButton"]

NEW = 2
NEW_LATE = 0
COMPLETE = 1
INCOMPLETE = 3
CANCELLED = 2
SKIPPED = 2


@dataclass
class TimepointStatusButton(AppointmentButton):
    labels: tuple[str, str, str] = field(default=("Start", "Visit Report", "Visit Report"))
    colors: tuple[str, str, str, str] = field(
        default=("warning", "success", "default", "danger")
    )

    @property
    def title(self) -> str:
        title = _(super().title)
        if self.model_obj.appt_status == SKIPPED_APPT:
            title = _("Skipped as per protocol")
        elif self.model_obj.appt_status == CANCELLED_APPT:
            title = _("Cancelled")
        elif self.model_obj.appt_status == COMPLETE_APPT:
            title = _("Done. All required forms submitted. Click to re-open")
        elif self.model_obj.appt_status == INCOMPLETE_APPT:
            title = _("Incomplete. Some forms not submitted. Click to re-open")
        elif self.model_obj.appt_status == NEW_APPT:
            title = _("Start appointment")
        if self.perms.view_only:
            title = _("%(title)s (view only)") % {"title": title}
        return title

    @property
    def label(self) -> str:
        label = None
        if self.model_obj.appt_status == SKIPPED_APPT:
            label = _("Skipped")
        elif self.model_obj.appt_status == CANCELLED_APPT:
            label = _("Cancelled")
        elif self.model_obj.appt_status == COMPLETE_APPT:
            label = _("Done")
        elif self.model_obj.appt_status == INCOMPLETE_APPT:
            label = _("Incomplete")
        elif self.model_obj.appt_status == NEW_APPT:
            label = _("Start")
        return label

    @property
    def color(self) -> str:
        color = super().color
        if self.model_obj.appt_status in [COMPLETE_APPT, SKIPPED_APPT]:
            color = self.colors[COMPLETE]  # success / gree
        elif self.model_obj.appt_status == CANCELLED_APPT:
            color = self.colors[CANCELLED]  # default / grey
        elif self.model_obj.appt_status == INCOMPLETE_APPT:
            color = self.colors[INCOMPLETE]  # default / grey
        elif self.model_obj.appt_status == NEW_APPT:
            if self.model_obj.appt_datetime <= get_utcnow():
                color = self.colors[NEW_LATE]  # warning / orange
            else:
                color = self.colors[NEW]
        return color

    @property
    def disabled(self) -> str:
        disabled = "disabled"
        if not self.model_obj and self.perms.add:
            disabled = ""
        else:
            if self.perms.change or self.perms.view:
                disabled = ""
        return disabled



from django.contrib.auth.mixins import LoginRequiredMixin
from django.http.response import HttpResponseRedirect
from django.urls.base import reverse
from django.views.generic.base import View
from edc_dashboard.url_names import url_names
from edc_label.printers_mixin import PrintersMixin


class BaseRequisitionView(LoginRequiredMixin, PrintersMixin, View):
    success_url_name = "subject_dashboard_url"

    def get_success_url(self):
        success_url = url_names.get(self.success_url_name)
        return success_url

    def get(self, request, *args, **kwargs):
        url = reverse("edc_lab_dashboard:home_url")
        return HttpResponseRedirect(url)

    def head(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def options(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def put(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def patch(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)



from edc_sites.single_site import SingleSite

__all__ = ["all_sites"]

suffix = "clinicedc.org"

all_sites = [
    SingleSite(
        110,
        "capetown",
        title="UCT: Khayelitsha and Mitchell’s Plain (Cape Town)",
        country="south_africa",
        country_code="sa",
        language_codes=["en"],
        domain=f"capetown.sa.{suffix}",
    ),
    SingleSite(
        120,
        "baragwanath",
        title="Wits: Chris Hani Baragwanath (Soweto)",
        country="south_africa",
        country_code="sa",
        language_codes=["en"],
        domain=f"baragwanath.sa.{suffix}",
    ),
    SingleSite(
        130,
        "helen_joseph",
        title="Wits: Helen Joseph (Johannesburg)",
        country="south_africa",
        country_code="sa",
        language_codes=["en"],
        domain=f"helen-joseph.sa.{suffix}",
    ),
    SingleSite(
        140,
        "tshepong",
        title="Wits: Tshepong (Klerksdorp)",
        country="south_africa",
        country_code="sa",
        language_codes=["en"],
        domain=f"tshepong.sa.{suffix}",
    ),
]



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypeVar

from django.utils.translation import gettext as _
from edc_appointment.constants import IN_PROGRESS_APPT
from edc_constants.constants import COMPLETE
from edc_view_utils import CHANGE, VIEW, NextQuerystring
from edc_visit_tracking.view_utils import RelatedVisitButton

if TYPE_CHECKING:
    from edc_visit_tracking.model_mixins import VisitModelMixin

    VisitModel = TypeVar("VisitModel", bound=VisitModelMixin)


__all__ = ["GotToFormsButton"]


@dataclass
class GotToFormsButton(RelatedVisitButton):
    """A button displayed on the subject dashboard of appointments
    that takes the user to the subject dashboard of CRFs and
    requisitions for the selected timepoint.
    """

    colors: tuple[str, str, str] = field(default=("primary", "primary", "default"))

    @property
    def label(self) -> str:
        return _("Forms")

    @property
    def title(self) -> str:
        return _("Go to CRFs and Requisitions")

    def color(self) -> str:
        """Shows as blue to direct user to go to the subject
        dashboard for this timepoint to edit CRFs and
        requisitions.

        Note:
            This button is relevant when the appointment is in
            progress. The related_visit document_status is set to
            COMPLETE when the related_visit is saved or re-saved.
            See also note on RelatedVisitButton.
        """
        color = self.colors[VIEW]
        if self.model_obj and self.appointment.appt_status == IN_PROGRESS_APPT:
            if self.model_obj.document_status == COMPLETE:
                color = self.colors[CHANGE]  # primary / blue
            else:
                color = self.colors[VIEW]  # default / grey
        return color

    @property
    def url(self) -> str:
        nq = NextQuerystring(
            next_url_name=self.next_url_name,
            reverse_kwargs=self.reverse_kwargs,
            extra_kwargs=self.extra_kwargs,
        )

        url = "?".join([f"{nq.next_url}", nq.querystring])
        return url

    @property
    def disabled(self) -> str:
        """Is only enabled if the appointment is set to
        IN_PROGRESS_APPT and the related_visit has been saved/resaved
        since appointment was set to IN PROGRESS.
        """
        disabled = "disabled"
        if (
            self.appointment.appt_status == IN_PROGRESS_APPT
            and self.model_obj.document_status == COMPLETE
            and any([self.perms.add, self.perms.change, self.perms.view])
        ):
            disabled = ""
        return disabled

    @property
    def label_fa_icon(self) -> str:
        if self.disabled:
            return ""
        return "fas fa-share"

    @property
    def fa_icon(self) -> str:
        return ""



from __future__ import annotations

from collections import namedtuple
from typing import TYPE_CHECKING, Type, TypeVar

from dateutil.relativedelta import relativedelta
from django import template
from django.apps import apps as django_apps
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.urls import reverse
from edc_appointment.constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    INCOMPLETE_APPT,
    NEW_APPT,
    SKIPPED_APPT,
)
from edc_appointment.models import Appointment
from edc_appointment.utils import (
    get_appointment_model_cls,
    get_unscheduled_appointment_url,
)
from edc_appointment.view_utils import AppointmentButton
from edc_auth.constants import AUDITOR_ROLE
from edc_metadata import KEYED, REQUIRED
from edc_metadata.metadata_helper import MetadataHelper
from edc_utils import get_utcnow
from edc_view_utils import PrnButton, render_history_and_query_buttons
from edc_visit_tracking.view_utils import RelatedVisitButton

from ..view_utils import (
    CrfButton,
    GotToFormsButton,
    RequisitionButton,
    SubjectConsentDashboardButton,
    SubjectConsentListboardButton,
    SubjectScreeningButton,
    TimepointStatusButton,
)

if TYPE_CHECKING:
    from edc_consent.model_mixins import ConsentModelMixin
    from edc_metadata.models import CrfMetadata
    from edc_registration.models import RegisteredSubject
    from edc_screening.model_mixins import ScreeningModelMixin
    from edc_visit_schedule.models import VisitSchedule as VisitScheduleModel
    from edc_visit_schedule.schedule import Schedule
    from edc_visit_schedule.visit_schedule import VisitSchedule
    from edc_visit_tracking.model_mixins import VisitModelMixin

    ConsentModel = TypeVar("ConsentModel", bound=ConsentModelMixin)
    ScreeningModel = TypeVar("ScreeningModel", bound=ScreeningModelMixin)
    VisitModel = TypeVar("VisitModel", bound=VisitModelMixin)

__all__ = [
    "appointment_in_progress",
    "render_appointment_status_icon",
    "print_requisition_popover",
    "render_prn_button",
    "render_appointment_button",
    "render_crf_button_group",
    "render_gotoforms_button",
    "requisition_panel_actions",
    "render_crf_totals",
    "render_subject_consent_dashboard_button",
    "render_unscheduled_appointment_button",
]

register = template.Library()


class SubjectDashboardExtrasError(Exception):
    pass


@register.inclusion_tag("edc_subject_dashboard/appointment_in_progress.html")
def appointment_in_progress(
    subject_identifier: str = None,
    visit_schedule: VisitSchedule = None,
    schedule: Schedule = None,
) -> dict[str, str]:
    """Returns the context with the visit code of the appointment in
    progress.
    """
    try:
        appointment = get_appointment_model_cls().objects.get(
            subject_identifier=subject_identifier,
            visit_schedule_name=visit_schedule.name,
            schedule_name=schedule.name,
            appt_status=IN_PROGRESS_APPT,
        )
    except ObjectDoesNotExist:
        visit_code = None
    except MultipleObjectsReturned:
        qs = get_appointment_model_cls().objects.filter(
            subject_identifier=subject_identifier,
            visit_schedule_name=visit_schedule.name,
            schedule_name=schedule.name,
            appt_status=IN_PROGRESS_APPT,
        )
        visit_code = ", ".join([obj.visit_code for obj in qs])
    else:
        visit_code = appointment.visit_code
    return dict(visit_code=visit_code)


@register.inclusion_tag(
    "edc_subject_dashboard/requisition_panel_actions.html",
    takes_context=True,
)
def requisition_panel_actions(context, requisitions=None):
    try:
        requisition_metadata = requisitions[0]
    except IndexError:
        context["verify_disabled"] = None
    else:
        app_label, model_name = requisition_metadata.model.split(".")
        context["verify_disabled"] = (
            None
            if context["user"].has_perm(f"{app_label}.change_{model_name}")
            else "disabled"
        )
    appointment = context.get("appointment")
    scanning = context.get("scanning")
    autofocus = "autofocus" if scanning else ""
    context["appointment"] = str(appointment.id)
    context["autofocus"] = autofocus
    return context


@register.inclusion_tag(
    "edc_subject_dashboard/print_requisition_popover.html",
    takes_context=True,
)
def print_requisition_popover(context):
    C = namedtuple("Consignee", "pk name")
    consignees = []
    for consignee in django_apps.get_model("edc_lab.Consignee").objects.all():
        consignees.append(C(str(consignee.pk), consignee.name))
    context["consignees"] = consignees
    return context


@register.inclusion_tag("edc_subject_dashboard/appointment_status.html")
def render_appointment_status_icon(appt_status: str = None) -> dict[str, str]:
    return dict(
        appt_status=appt_status,
        NEW_APPT=NEW_APPT,
        IN_PROGRESS_APPT=IN_PROGRESS_APPT,
        INCOMPLETE_APPT=INCOMPLETE_APPT,
        COMPLETE_APPT=COMPLETE_APPT,
        CANCELLED_APPT=CANCELLED_APPT,
        SKIPPED_APPT=SKIPPED_APPT,
    )


@register.inclusion_tag(
    "edc_subject_dashboard/dashboard/crf_totals.html",
)
def render_crf_totals(appointment: Appointment = None) -> dict[str, bool | int]:
    helper = MetadataHelper(appointment)
    skipped: bool = False
    show_totals: bool = False
    overdue: bool = False
    complete: bool = False
    num_keyed: int = 0
    num_total: int = 0
    if appointment.appt_status == SKIPPED_APPT:
        skipped = True
    elif (
        appointment.appt_status == NEW_APPT
        and appointment.appt_datetime.date() < get_utcnow().date()
    ):
        overdue = True
    else:
        crf_keyed = helper.get_crf_metadata_by(entry_status=KEYED).count()
        requisition_keyed = helper.get_requisition_metadata_by(entry_status=KEYED).count()
        crf_total = helper.get_crf_metadata_by(entry_status=[REQUIRED, KEYED]).count()
        requisition_total = helper.get_requisition_metadata_by(
            entry_status=[REQUIRED, KEYED]
        ).count()
        num_keyed = crf_keyed + requisition_keyed
        num_total = crf_total + requisition_total
        if appointment.related_visit:
            show_totals = False if num_keyed != 0 and num_keyed == num_total else True
        complete = num_keyed != 0 and num_keyed == num_total
    return dict(
        show_totals=show_totals,
        skipped=skipped,
        complete=complete,
        overdue=overdue,
        keyed=num_keyed,
        total=num_total,
    )


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/crf_button_group.html",
    takes_context=True,
)
def render_crf_button_group(
    context,
    model_obj: CrfMetadata = None,
    appointment: Appointment = None,
    registered_subject: RegisteredSubject = None,
    visit_schedule: VisitScheduleModel = None,
):
    """Prepare context data to render CRF, History, and Query
    dashboard buttons.

    For example, in the template:
        Where `crf` is an instance of CRFMetadata

        {% crf_button_group "subject_dashboard_url" crf
           appointment registered_subject
           visit_schedule_model_obj %}
    """
    # if still using deprecated ModelWrapper, get model instance
    # from model_wrapper
    appointment = getattr(appointment, "object", appointment)
    crf_btn = CrfButton(
        metadata_model_obj=model_obj,
        appointment=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    history_btn, query_btn = render_history_and_query_buttons(
        context, model_obj, appointment, registered_subject, visit_schedule
    )
    return dict(buttons=[crf_btn, history_btn, query_btn])


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/crf_button_group.html",
    takes_context=True,
)
def render_requisition_button_group(
    context,
    model_obj: CrfMetadata = None,
    appointment: Appointment = None,
    registered_subject: RegisteredSubject = None,
    visit_schedule: VisitScheduleModel = None,
):
    # if still using deprecated ModelWrapper, get model instance
    # from model_wrapper
    appointment = getattr(appointment, "object", appointment)
    requisition_btn = RequisitionButton(
        metadata_model_obj=model_obj,
        appointment=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    history_btn, query_btn = render_history_and_query_buttons(
        context, model_obj, appointment, registered_subject, visit_schedule
    )
    return dict(buttons=[requisition_btn, history_btn, query_btn])


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/prn_button.html",
    takes_context=True,
)
def render_prn_button(context, model_obj, model_name: str) -> dict:
    # TODO: is this used?
    model_cls = django_apps.get_model(model_name)
    btn = PrnButton(
        model_obj=model_obj,
        model_cls=model_cls,
        subject_identifier=context.get("subject_identifier"),
        next_url_name=context.get("next_url_name"),
        user=context["request"].user,
        current_site=context["request"].site,
        request=context["request"],
    )
    return dict(btn=btn)


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/appointment_button.html",
    takes_context=True,
)
def render_appointment_button(context, appointment: Appointment = None):
    # if still using deprecated ModelWrapper, get model instance
    # from model_wrapper
    appointment = getattr(appointment, "object", appointment)
    appointment_btn = AppointmentButton(
        model_obj=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    return {"btn": appointment_btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/appointment_button.html",
    takes_context=True,
)
def render_related_visit_button(context, appointment: Appointment = None):
    # if still using deprecated ModelWrapper, get model instance
    # from model_wrapper
    appointment = getattr(appointment, "object", appointment)
    related_visit: VisitModel = appointment.related_visit
    related_visit_model_cls: Type[VisitModel] = appointment.related_visit_model_cls()
    btn = RelatedVisitButton(
        model_obj=related_visit,
        model_cls=related_visit_model_cls,
        appointment=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    return {"btn": btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/forms_button.html",
    takes_context=True,
)
def render_gotoforms_button(context, appointment: Appointment = None):
    # if still using deprecated ModelWrapper, get model instance
    # from model_wrapper
    appointment: Appointment = getattr(appointment, "object", appointment)
    related_visit: VisitModel = appointment.related_visit
    related_visit_model_cls: Type[VisitModel] = appointment.related_visit_model_cls()
    btn = GotToFormsButton(
        model_obj=related_visit,
        model_cls=related_visit_model_cls,
        appointment=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    return {"btn": btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/appointment_button.html",
    takes_context=True,
)
def render_timepoint_status_button(context, appointment: Appointment = None):
    btn = TimepointStatusButton(
        model_obj=appointment,
        user=context["user"],
        current_site=context["request"].site,
    )
    return {"btn": btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/subject_consent_button.html",
    takes_context=True,
)
def render_subject_listboard_consent_button(
    context,
    subject_screening: ScreeningModel = None,
    next_url_name: str | None = None,
):
    """A subject consent button to appear on the subject listboard.

    Consent is added from the listboard
    """
    btn = SubjectConsentListboardButton(
        screening_obj=subject_screening,
        user=context["user"],
        current_site=context["request"].site,
        next_url_name=next_url_name,
    )
    return {"btn": btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/subject_consent_button.html",
    takes_context=True,
)
def render_subject_consent_dashboard_button(
    context,
    consent: ConsentModel = None,
    appointment: Appointment = None,
    next_url_name: str | None = None,
):
    """A subject consent button to appear on the subject dashboard
    sidebar.

    Assumes the consent model instance exists.
    """
    btn = SubjectConsentDashboardButton(
        model_obj=consent,
        user=context["user"],
        current_site=context["request"].site,
        appointment=appointment,
        next_url_name=next_url_name,
    )
    return {"btn": btn}


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/unscheduled_appointment_button.html",
    takes_context=True,
)
def render_unscheduled_appointment_button(
    context, appointment: Appointment = None, view_appointment: bool = None
):
    show_button = False
    anchor_id: str | None = None
    url: str | None = None
    disabled = "disabled"
    title: str | None = None
    any_in_progress = appointment.__class__.objects.filter(
        appt_status=IN_PROGRESS_APPT,
        subject_identifier=appointment.subject_identifier,
        visit_schedule_name=appointment.visit_schedule_name,
        schedule_name=appointment.schedule_name,
    ).exists()
    if (
        not any_in_progress
        and appointment.appt_status in [INCOMPLETE_APPT, COMPLETE_APPT]
        and appointment
        and appointment.site.id == context["request"].site.id
        and appointment.relative_next
        and (
            appointment.appt_datetime.date() + relativedelta(days=1)
            != appointment.relative_next.appt_datetime.date()
        )
    ):
        show_button = True
        anchor_id = (
            f"unscheduled_appt_btn_{appointment.visit_code}_"
            f"{appointment.visit_code_sequence}"
        )
        if view_appointment and appointment.site.id == context["request"].site.id:
            url = get_unscheduled_appointment_url(appointment)
        disabled = "disabled" if not url else ""
        if view_appointment and appointment.site.id == context["request"].site.id:
            title = "" if disabled else "Edit appointment"
        else:
            title = "No permission to edit"
    return dict(
        show_button=show_button,
        anchor_id=anchor_id,
        appointment=appointment,
        view_appointment=view_appointment,
        url=url,
        disabled=disabled,
        title=title,
        INCOMPLETE_APPT=INCOMPLETE_APPT,
        COMPLETE_APPT=COMPLETE_APPT,
    )


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/forms_button.html",
    takes_context=True,
)
def render_screening_button(context, subject_screening) -> dict:
    btn = SubjectScreeningButton(
        user=context["request"].user,
        model_obj=subject_screening,
        next_url_name="screening_listboard_url",
        current_site=context["request"].site,
    )

    return dict(btn=btn)


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/subject_schedule_button.html",
)
def render_subject_schedule_button(
    subject_dashboard_url: str, subject_identifier: str
) -> dict:
    url = reverse(subject_dashboard_url, kwargs=dict(subject_identifier=subject_identifier))
    return dict(url=url, subject_identifier=subject_identifier)


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/refresh_appointments_button.html",
    takes_context=True,
)
def render_refresh_appointments_button(
    context,
    subject_identifier: str = None,
    visit_schedule_name: str = None,
    schedule_name: str = None,
) -> dict:
    if context["request"].user.userprofile.is_multisite_viewer:
        url = None
    elif context["request"].user.userprofile.roles.filter(name=AUDITOR_ROLE):
        url = None
    else:
        url = reverse(
            "edc_subject_dashboard:refresh_appointments_url",
            kwargs=dict(
                subject_identifier=subject_identifier,
                visit_schedule_name=visit_schedule_name,
                schedule_name=schedule_name,
            ),
        )
    return dict(url=url)


@register.inclusion_tag(
    "edc_subject_dashboard/buttons/refresh_data_collection_schedule_button.html",
    takes_context=True,
)
def render_refresh_data_collection_schedule_button(
    context,
    related_visit_id: str = None,
    visit_schedule_name: str = None,
    schedule_name: str = None,
) -> dict:
    if context["request"].user.userprofile.is_multisite_viewer:
        url = None
    elif context["request"].user.userprofile.roles.filter(name=AUDITOR_ROLE):
        url = None
    elif not related_visit_id:
        url = None
    else:
        url = reverse(
            "edc_subject_dashboard:refresh_metadata_actions_url",
            kwargs=dict(
                related_visit_id=related_visit_id,
                visit_schedule_name=visit_schedule_name,
                schedule_name=schedule_name,
            ),
        )
    return dict(url=url)



#!/usr/bin/env python
import sys
from pathlib import Path

from edc_test_settings.default_test_settings import DefaultTestSettings

app_name = "edc_subject_dashboard"
base_dir = Path(__file__).absolute().parent.parent.parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=str(base_dir / "edc_subject_dashboard" / "tests" / "etc"),
    SILENCED_SYSTEM_CHECKS=[
        "sites.E101",
        "edc_navbar.E002",
        "edc_navbar.E003",
        "edc_sites.E001",
        "edc_sites.E002",
    ],
    SUBJECT_VISIT_MODEL="edc_visit_tracking.subjectvisit",
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    EDC_SITES_REGISTER_DEFAULT=False,
    add_dashboard_middleware=True,
    add_lab_dashboard_middleware=True,
    ROOT_URLCONF="subject_dashboard_app.urls",
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_extensions",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "multisite",
        "edc_action_item.apps.AppConfig",
        "edc_adverse_event.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_auth.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_data_manager.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_export.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_form_runners.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_lab_dashboard.apps.AppConfig",
        "edc_label.apps.AppConfig",
        "edc_locator.apps.AppConfig",
        "edc_listboard.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_review_dashboard.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_model_admin.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "adverse_event_app.apps.AppConfig",
        "subject_dashboard_app.apps.AppConfig",
        "edc_appconfig.apps.AppConfig",
    ],
).settings

for k, v in project_settings.items():
    setattr(sys.modules[__name__], k, v)



from django.urls import include, path, re_path
from django.views.generic import RedirectView
from edc_dashboard.views import AdministrationView
from edc_protocol.research_protocol_config import ResearchProtocolConfig
from edc_utils.paths_for_urlpatterns import paths_for_urlpatterns

from edc_subject_dashboard.views import SubjectDashboardView as BaseSubjectDashboardView


class SubjectDashboardView(BaseSubjectDashboardView):
    navbar_name = "edc_subject_dashboard"
    visit_model = "edc_visit_tracking.subjectvisit"


app_name = "subject_dashboard_app"

urlpatterns = [
    path("accounts/", include("edc_auth.urls_for_accounts", namespace="auth")),
    path("administration/", AdministrationView.as_view(), name="administration_url"),
    *paths_for_urlpatterns("edc_auth"),
    *paths_for_urlpatterns("edc_action_item"),
    *paths_for_urlpatterns("edc_consent"),
    *paths_for_urlpatterns("edc_adverse_event"),
    *paths_for_urlpatterns("edc_device"),
    *paths_for_urlpatterns("edc_dashboard"),
    *paths_for_urlpatterns("edc_label"),
    *paths_for_urlpatterns("edc_lab"),
    *paths_for_urlpatterns("edc_lab_dashboard"),
    *paths_for_urlpatterns("edc_dashboard"),
    *paths_for_urlpatterns("edc_subject_dashboard"),
    *paths_for_urlpatterns("edc_review_dashboard"),
    *paths_for_urlpatterns("edc_export"),
    *paths_for_urlpatterns("edc_data_manager"),
    *paths_for_urlpatterns("edc_visit_tracking"),
    *paths_for_urlpatterns("edc_protocol"),
    *paths_for_urlpatterns("edc_visit_schedule"),
    *SubjectDashboardView.urls(
        # namespace="subject_dashboard_app",
        label="subject_dashboard",
        identifier_pattern=ResearchProtocolConfig().subject_identifier_pattern,
    ),
    path("i18n/", include("django.conf.urls.i18n")),
    re_path(".", RedirectView.as_view(url="/"), name="home_url"),
]



from datetime import date

from django.db import models
from django.db.models.deletion import PROTECT
from edc_appointment.models import Appointment
from edc_consent.managers import ConsentObjectsByCdefManager, CurrentSiteByCdefManager
from edc_crf.model_mixins import CrfModelMixin
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_lab.models import Panel
from edc_model.models import BaseUuidModel
from edc_registration.model_mixins import UpdatesOrCreatesRegistrationModelMixin
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow
from edc_visit_schedule.model_mixins import (
    OffScheduleModelMixin,
    OnScheduleModelMixin,
    VisitCodeFieldsModelMixin,
)
from edc_visit_tracking.models import SubjectVisit


class SubjectConsent(
    SiteModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    UpdatesOrCreatesRegistrationModelMixin,
    BaseUuidModel,
):
    consent_datetime = models.DateTimeField(default=get_utcnow)

    version = models.CharField(max_length=25, default="1")

    identity = models.CharField(max_length=25)

    dob = models.DateField(default=date(1995, 1, 1))


class SubjectConsentV1(SubjectConsent):
    objects = ConsentObjectsByCdefManager()
    on_site = CurrentSiteByCdefManager()

    class Meta:
        proxy = True


class OnSchedule(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):
    pass


class OffSchedule(SiteModelMixin, OffScheduleModelMixin, BaseUuidModel):
    pass


class SubjectRequisition(
    SiteModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    VisitCodeFieldsModelMixin,
    BaseUuidModel,
):
    @classmethod
    def related_visit_model_attr(cls):
        return "subject_visit"

    subject_visit = models.ForeignKey(SubjectVisit, on_delete=models.PROTECT, related_name="+")

    panel = models.ForeignKey(Panel, on_delete=PROTECT)

    class Meta(BaseUuidModel.Meta):
        pass


class TestModel(models.Model):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)


class BadSubjectVisit(models.Model):
    appointment = models.ForeignKey(Appointment, on_delete=PROTECT)

    subject_identifier = models.CharField(max_length=25)

    report_datetime = models.DateTimeField(default=get_utcnow)


class OnScheduleTwo(SiteModelMixin, OnScheduleModelMixin, BaseUuidModel):
    pass


class OffScheduleTwo(SiteModelMixin, OffScheduleModelMixin, BaseUuidModel):
    pass


class CrfOne(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)

    f2 = models.CharField(max_length=50, null=True)

    f3 = models.CharField(max_length=50, null=True)


class CrfTwo(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)


class CrfThree(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)


class CrfFour(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)


class CrfFive(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)



from django.test import TestCase, override_settings
from django.views.generic.base import ContextMixin
from edc_appointment.view_mixins import AppointmentViewMixin
from edc_consent import site_consents
from edc_locator.exceptions import SubjectLocatorViewMixinError
from edc_locator.view_mixins import SubjectLocatorViewMixin
from edc_sites.utils import get_site_model_cls
from edc_sites.view_mixins import SiteViewMixin
from edc_test_utils.get_httprequest_for_tests import get_request_object_for_tests
from edc_utils import get_utcnow
from edc_visit_schedule.site_visit_schedules import site_visit_schedules
from edc_visit_tracking.constants import SCHEDULED
from edc_visit_tracking.models import SubjectVisit

from edc_subject_dashboard.view_mixins import (
    RegisteredSubjectViewMixin,
    SubjectVisitViewMixin,
    SubjectVisitViewMixinError,
)
from subject_dashboard_app.consents import consent_v1
from subject_dashboard_app.models import (
    Appointment,
    BadSubjectVisit,
    SubjectConsent,
    TestModel,
)

from .test_case_mixin import TestCaseMixin


class DummyModelWrapper:
    def __init__(self, **kwargs):
        pass


@override_settings(SITE_ID=110)
class TestViewMixins(TestCaseMixin, TestCase):
    def setUp(self):
        site_consents.registry = {}
        site_consents.register(consent_v1)
        self.subject_identifier = "101-108987-0"
        self.current_site = get_site_model_cls().objects.get_current()
        self.subject_identifier = "101-1234567-0"
        subject_consent = SubjectConsent.objects.create(
            subject_identifier=self.subject_identifier, consent_datetime=get_utcnow()
        )
        _, self.schedule = site_visit_schedules.get_by_onschedule_model(
            "subject_dashboard_app.onschedule"
        )
        self.schedule.put_on_schedule(
            subject_identifier=self.subject_identifier,
            onschedule_datetime=subject_consent.consent_datetime,
        )

        self.appointment = Appointment.objects.get(visit_code="1000")
        self.subject_visit = SubjectVisit.objects.create(
            appointment=self.appointment,
            subject_identifier=self.subject_identifier,
            visit_schedule_name="visit_schedule1",
            schedule_name="schedule1",
            visit_code="1000",
            reason=SCHEDULED,
        )
        self.bad_subject_visit = BadSubjectVisit.objects.create(
            appointment=self.appointment, subject_identifier=self.subject_identifier
        )
        self.test_model = TestModel.objects.create(subject_visit=self.subject_visit)

    def test_subject_visit_incorrect_relation(self):
        """Asserts raises if relation is not one to one."""

        class MySubjectVisitViewMixin(
            SubjectVisitViewMixin,
            RegisteredSubjectViewMixin,
            ContextMixin,
        ):
            visit_attr = "badsubjectvisit"

        mixin = MySubjectVisitViewMixin()
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        mixin.request = get_request_object_for_tests(self.user)
        self.assertRaises(SubjectVisitViewMixinError, mixin.get_context_data)

    def test_subject_locator_raises_on_bad_model(self):
        class MySubjectLocatorViewMixin(
            SiteViewMixin,
            SubjectLocatorViewMixin,
            RegisteredSubjectViewMixin,
            AppointmentViewMixin,
            ContextMixin,
        ):
            subject_locator_model = "blah.blahblah"

        mixin = MySubjectLocatorViewMixin()
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        mixin.request = get_request_object_for_tests(self.user)
        self.assertRaises(LookupError, mixin.get_context_data)

    def test_subject_locator_ok(self):
        class MySubjectLocatorViewMixin(
            SubjectLocatorViewMixin,
            RegisteredSubjectViewMixin,
            AppointmentViewMixin,
            ContextMixin,
        ):
            subject_locator_model = "edc_locator.subjectlocator"

        mixin = MySubjectLocatorViewMixin()
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        mixin.request = get_request_object_for_tests(self.user)
        try:
            mixin.get_context_data()
        except SubjectLocatorViewMixinError as e:
            self.fail(e)



from django.contrib.admin import AdminSite as DjangoAdminSite
from edc_locator.models import SubjectLocator

from .models import Appointment, SubjectConsent, SubjectVisit


class AdminSite(DjangoAdminSite):
    pass


subject_dashboard_app_admin = AdminSite(name="subject_dashboard_app_admin")

subject_dashboard_app_admin.register(Appointment)
subject_dashboard_app_admin.register(SubjectConsent)
subject_dashboard_app_admin.register(SubjectLocator)
subject_dashboard_app_admin.register(SubjectVisit)
